﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Diagnostics;


namespace PythonIDE {
    public partial class Form1 : Form, ICloseWindow {

        

        private int _fontsize = 11;

        public Form1() {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e) {
            setTabWidth(ConstantsVariablen.Tabwidth);
            int f = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "MainFontSize");
            if (f > 7) {
                _fontsize = f;
                clearFonts();
                setFont(f);
                switch (f) {
                    case 8: mnFontsize8pg.Checked = true; break;
                    case 10: mnFontsize10pg.Checked = true; break;
                    case 12: mnFontsize12pg.Checked = true; break;
                    case 14: mnFontsize14pg.Checked = true; break;
                    case 16: mnFontsize16pg.Checked = true; break;
                    case 18: mnFontsize18pg.Checked = true; break;
                    case 20: mnFontsize20pg.Checked = true; break;
                    case 22: mnFontsize22pg.Checked = true; break;
                    case 24: mnFontsize24pg.Checked = true; break;
                    case 26: mnFontsize26pg.Checked = true; break;
                    case 28: mnFontsize28pg.Checked = true; break;
                    case 30: mnFontsize30pg.Checked = true; break;
                }
            }
            int t = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "Main_Top");
            int l = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "Main_Left");
            int w = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "Main_Width");
            int h = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "Main_Height");
            if (t >= 0 && l >= 0 && w >= 100 && h >= 100) {
                if (l + w <= Screen.PrimaryScreen.Bounds.Width) {
                    Left = l;
                    Width = w;
                }
                if (t + h <= Screen.PrimaryScreen.Bounds.Height) {
                    Top = t;
                    Height = h;
                }
            }

            setEnabled(false);
            int whiteScreenBlackFont = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "WhiteScreenBlackFont");
            if (whiteScreenBlackFont == 0) {
                setColors();
            }
        }

        public void CloseWindow(Form mdiChild) {
            int anz = 0;
            foreach (Form form in this.MdiChildren) {
                if (form != mdiChild) anz++;                
            }
            setEnabled(anz > 0);
        }

        private void setEnabled(bool enabled) {
            bnSave.Enabled = enabled;
            bnSaveAll.Enabled = enabled;
            mnSave.Enabled = enabled;
            mnSaveAll.Enabled = enabled;
        }

        private void loadFile(String filename) {
            WEditor childForm = new WEditor(this, filename);
            childForm.MdiParent = this;
            childForm.Show();
            childForm.setFont(_fontsize);
            childForm.setTabWidth(ConstantsVariablen.Tabwidth);
            setEnabled(this.MdiChildren.Length > 0);
        }

        private void mnNew_Click(object sender, EventArgs e) {
            loadFile("");
        }

        private void mnOpen_Click(object sender, EventArgs e) {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = ConstantsVariablen.WorkingDir;
            if (ActiveMdiChild != null) {
                WEditor form = (WEditor)ActiveMdiChild;
                if (form.Filename.Length > 0) {
                    openFileDialog.InitialDirectory = Basis.getPath(form.Filename);
                }
            }

            openFileDialog.Multiselect = true;

            openFileDialog.Filter = "Pythondateien (*.py)|*.py|Alle Dateien (*.*)|*.*";
            if (openFileDialog.ShowDialog(this) == DialogResult.OK) {
                foreach (string filename in openFileDialog.FileNames) {
                    loadFile(filename);
                }
            }
        }

        private void mnSave_Click(object sender, EventArgs e) {
            if (ActiveMdiChild != null) {
                WEditor form = (WEditor)ActiveMdiChild;
                if (form.Filename.Length == 0) {
                    mnSaveAs_Click(sender, e);
                }
                else {
                    form.saveFile();
                }
            }
        }

        private void mnSaveAll_Click(object sender, EventArgs e) {
            foreach (Form form in this.MdiChildren) {
                WEditor editor = (WEditor)form;
                editor.saveFile();
            }
        }


        private void mnSaveAs_Click(object sender, EventArgs e) {
            WEditor form = (WEditor)ActiveMdiChild;
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            if (form.Filename.Length > 0) {
                saveFileDialog.InitialDirectory = Basis.getPath(form.Filename);
            }
            else {
                saveFileDialog.InitialDirectory = ConstantsVariablen.WorkingDir;
            }
            saveFileDialog.Filter = "Pythonateien (*.py)|*.py|Alle Dateien (*.*)|*.*";
            saveFileDialog.FileName = Basis.getFilename(form.Filename);
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK) {
                string filename = saveFileDialog.FileName;
                form.Filename = filename;
                form.saveFile();
            }
        }

        private void mnClose_Click(object sender, EventArgs e) {
            Close();
        }

        private void mnTabWidth1_Click(object sender, EventArgs e) {
            setTabWidth(1);
        }

        private void mnTabWidth2_Click(object sender, EventArgs e) {
            setTabWidth(2);
        }

        private void mnTabWidth3_Click(object sender, EventArgs e) {
            setTabWidth(3);
        }

        private void mnTabWidth4_Click(object sender, EventArgs e) {
            setTabWidth(4);
        }

        private void mnTabWidth5_Click(object sender, EventArgs e) {
            setTabWidth(5);
        }

        private void mnTabWidth6_Click(object sender, EventArgs e) {
            setTabWidth(6);
        }

        private void mnTabWidth7_Click(object sender, EventArgs e) {
            setTabWidth(7);
        }

        private void mnTabWidth8_Click(object sender, EventArgs e) {
            setTabWidth(8);
        }

        private void setTabWidth(int width) {
            ConstantsVariablen.Tabwidth = width; // *FAKTORTAB;
            foreach (Form form in this.MdiChildren) {
                WEditor editor = (WEditor)form;
                editor.setTabWidth(ConstantsVariablen.Tabwidth);
            }
            mnTabWidth1.Checked = false;
            mnTabWidth2.Checked = false;
            mnTabWidth3.Checked = false;
            mnTabWidth4.Checked = false;
            mnTabWidth5.Checked = false;
            mnTabWidth6.Checked = false;
            mnTabWidth7.Checked = false;
            mnTabWidth8.Checked = false;
            switch (width) {
                case 1: mnTabWidth1.Checked = true; break;
                case 2: mnTabWidth2.Checked = true; break;
                case 3: mnTabWidth3.Checked = true; break;
                case 4: mnTabWidth4.Checked = true; break;
                case 5: mnTabWidth5.Checked = true; break;
                case 6: mnTabWidth6.Checked = true; break;
                case 7: mnTabWidth7.Checked = true; break;
                case 8: mnTabWidth8.Checked = true; break;
            }
        }

        private void bnSetTabWidth_Click(object sender, EventArgs e) {
            uint tabwidth;
            if (uint.TryParse(tTabWidth.Text, out tabwidth)) {
                foreach (Form form in this.MdiChildren) {
                    WEditor editor = (WEditor)form;
                    editor.setTabWidth2(tabwidth);
                }
            }
        }


        private void clearFonts() {
            mnFontsize8pg.Checked = false;
            mnFontsize10pg.Checked = false;
            mnFontsize12pg.Checked = false;
            mnFontsize14pg.Checked = false;
            mnFontsize16pg.Checked = false;
            mnFontsize18pg.Checked = false;
            mnFontsize20pg.Checked = false;
            mnFontsize22pg.Checked = false;
            mnFontsize24pg.Checked = false;
            mnFontsize26pg.Checked = false;
            mnFontsize28pg.Checked = false;
            mnFontsize30pg.Checked = false;
        }

        private void mnOptions_Click(object sender, EventArgs e) {
            WOptions dialog = new WOptions();
            dialog.ShowDialog();
            if (dialog.retcode) {
                foreach (WEditor form in MdiChildren) {
                    form.setColors();
                }
                setColors();
            }
        }

        public void setColors() {
            toolStrip.BackColor = ConstantsVariablen.BGcolor;
            bnStart.ForeColor = ConstantsVariablen.Fontcolor;

            statusStrip.ForeColor = ConstantsVariablen.Fontcolor;
            statusStrip.BackColor = ConstantsVariablen.BGcolor;
        }

        private void setFont(int size) {
            _fontsize = size;
            foreach (WEditor form in MdiChildren) {
                form.setFont(size);
            }
            Font f = new Font("Arial", size);
            mainFile.Font = f;
            mainHelp.Font = f;
            mainTools.Font = f;
            mainView.Font = f;
            mainWindow.Font = f;
            mainFont.Font = f;
            mainProgramm.Font = f;
            bnStart.Font = f;
            Basis.WriteRegInt(ConstantsVariablen.C_REG_ADR, "MainFontSize", size);
        }

        private void mnFontsize8pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize8pg.Checked = true;
            setFont(8);

        }

        private void mnFontsize10pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize10pg.Checked = true;
            setFont(10);

        }

        private void mnFontsize12pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize12pg.Checked = true;
            setFont(12);

        }

        private void mnFontsize14pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize12pg.Checked = true;
            setFont(14);

        }

        private void mnFontsize16pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize16pg.Checked = true;
            setFont(16);

        }

        private void mnFontsize18pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize18pg.Checked = true;
            setFont(18);

        }

        private void mnFontsize20pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize20pg.Checked = true;
            setFont(20);

        }

        private void mnFontsize22pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize22pg.Checked = true;
            setFont(22);

        }

        private void mnFontsize24pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize24pg.Checked = true;
            setFont(24);
        }

        private void mnFontsize26pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize26pg.Checked = true;
            setFont(26);
        }

        private void mnFontsize28pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize24pg.Checked = true;
            setFont(24);

        }

        private void mnFontsize30pg_Click(object sender, EventArgs e) {
            clearFonts();
            mnFontsize24pg.Checked = true;
            setFont(24);

        }

      

        private void mnCascade_Click(object sender, EventArgs e) {
            LayoutMdi(MdiLayout.Cascade);
        }

        private void mnTileVertical_Click(object sender, EventArgs e) {
            LayoutMdi(MdiLayout.TileVertical);
        }

        private void mnTileHorizontal_Click(object sender, EventArgs e) {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        private void mnCloseAll_Click(object sender, EventArgs e) {

        }

        private void mnArrangeIcons_Click(object sender, EventArgs e) {
            LayoutMdi(MdiLayout.ArrangeIcons);
        }

        private void mnProgInfo_Click(object sender, EventArgs e) {
            WProgInfo dialog = new WProgInfo();
            dialog.ShowDialog();
        }

        private void mnDosEbene_Click(object sender, EventArgs e) {
            if (ActiveMdiChild != null) {
                WEditor form = (WEditor)ActiveMdiChild;
                Process Prog = new Process();
                Prog.StartInfo.FileName = "cmd.exe";
                Prog.StartInfo.Arguments = "";
                if (form.Filename.Equals(""))
                    Prog.StartInfo.WorkingDirectory = Environment.CurrentDirectory;
                else
                    Prog.StartInfo.WorkingDirectory = Basis.getPath(form.Filename);
                Prog.Start();
            }
        }

        private void mnStartPythonScript(bool withCMD) {
            if (ActiveMdiChild != null) {
                WEditor form = (WEditor)ActiveMdiChild;
                if (form.Filename.Length == 0) {
                    mnSaveAs_Click(null, null);
                }

                if (form.Filename.Length > 0) {
                    form.start(withCMD);
                }
            }
        }

        private void mnStart_Click(object sender, EventArgs e) {
            mnStartPythonScript(false);
        }

        private void mnStartCMD_Click(object sender, EventArgs e) {
            mnStartPythonScript(true);
        }

      


        /*
CMD [/A | /U] [/Q] [/D] [/E:ON | /E:OFF] [/F:ON | /F:OFF] [/V:ON | /V:OFF]
    [[/S] [/C | /K] Zeichenfolge]

/C      Führt den Befehl in der Zeichenfolge aus und endet dann.
/K      Führt den Befehl in der Zeichenfolge aus und endet dann nicht.
/S      Bestimmt Behandlung von Zeichenfolgen nach /C oder /K (siehe unten).
/Q      Schaltet die Befehlsanzeige aus.
/D      Deaktiviert die Ausführung von AutoRun-Befehlen von der Registrierung.
/A      Ausgabe interner Befehle in eine Pipe oder Datei im ANSI-Format.
/U      Ausgabe interner Befehle in eine Pipe oder Datei im UNICODE-Format.
/T:fg   Legt die Hinter- bzw. Vordergrundfarben fest (siehe auch COLOR /?).
/E:ON   Aktiviert Befehlserweiterungen (siehe unten).
/E:OFF  Deaktiviert Befehlserweiterungen (siehe unten).
/F:ON   Aktiviert die Ergänzung von Datei- und Verzeichnisnamen.
/F:OFF  Deaktiviert die Ergänzung von Datei- und Verzeichnisnamen.
/V:ON   Aktiviert verzögerte Erweiterung von Variablen, ! wird dabei als
        Trennzeichen verwendet. Zum Beispiel wird mit /V:ON die Variable
        !var! zur Ausführungszeit erweitert. Im Gegensatz dazu wird bei
        Verwendung der Syntax var die Variable zum Zeitpunkt der Eingabe
        aufgelöst (Diese Werte können z. B. in einer FOR-Schleife
        unterschiedlich sein!).
/V:OFF  Deaktiviert die verzögerte Erweiterung von Variablen.  
*/
     

    }
}
