﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;

using System.Runtime.InteropServices;
using System.Diagnostics;


namespace PythonIDE {
    public partial class WEditor : Form {

        private String _filename = "";

        String search = "";
        Boolean ignoreCase = false;
        int position = 0;

        ICloseWindow parent = null;


        public String Filename {
            get { return _filename; }
            set {
                if (value.Length > 0) {
                    _filename = value;
                    this.Text = Basis.getFilename(_filename);  // get dateinamen ohne Pfad
                    lDateiname.Text = Basis.getFilename(_filename);
                    toolTip1.SetToolTip(lDateiname, _filename);
                }
            }
        }

        public WEditor(ICloseWindow parent, String filename) {
            InitializeComponent();
            _filename = filename;
            this.Text = "noname";
            this.parent = parent;
        }


        private void WEditor_Load(object sender, EventArgs e) {
            Width = 500;
            Height = 400;
            lRow.Text = "Row: ";
            lColumn.Text = "Col: ";
            lDateiname.Text = "Name: ";

            if (!_filename.Equals("")) {
                loadFilename(_filename);
                Filename = _filename;
                // this.Text = Basis.getFilename(_filename);
            }
            else {
                lDateiname.Text = "noname";
                editor.Text = "#! /usr/bin/env python3\r\n# coding=utf8\r\n";
            }
            //TopToolStripPanel.Dock = DockStyle.Fill;
            toolStripContainer1.Dock = DockStyle.Fill;
            editor.Font = new Font("Courier New", 11);
            tableLayoutPanel1.Dock = DockStyle.Fill;
            editor.Dock = DockStyle.Fill;
            flowLayoutPanel1.Dock = DockStyle.Fill;
            flowLayoutPanel1.AutoSize = true;

            lColumn.Margin = new Padding(8, 0, 4, 0);  // left top right bottom
            lDateiname.Margin = new Padding(8, 0, 4, 0);  // left top right bottom
            int whiteScreenBlackFont = Basis.ReadRegInt(ConstantsVariablen.C_REG_ADR, "WhiteScreenBlackFont");
            if (whiteScreenBlackFont == 0) {
                setColors();
            }

            //setHelp();

        }

        public void setFont(int size) {
            editor.Font = new Font("Courier New", size);
            //editor.Font = new Font("Segoe UI", size);
            Font f = new Font("Arial", size);
            mainEdit.Font = f;
            mainPython.Font = f;

            lRow.Font = f;
            lColumn.Font = f;
            lDateiname.Font = f;

            //int h = lRow.Height;
            setTabWidth(ConstantsVariablen.Tabwidth);
        }

      


        // using System.Runtime.InteropServices;
        private const int EM_SETTABSTOPS = 0x00CB;

        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        // public static extern IntPtr SendMessage(IntPtr h, int msg, int wParam, int[] lParam);

        public static extern IntPtr SendMessage(IntPtr hWnd, UInt32 Msg, IntPtr wParam, ref int lParam);

        public void setTabWidth(int tabWidth) {
            int w;
            Graphics graphics = editor.CreateGraphics();
            var characterWidth = graphics.MeasureString("m", editor.Font).Width;
            w = (int)(tabWidth * characterWidth / 4);
            // w = tabWidth;
            SendMessage(editor.Handle, EM_SETTABSTOPS, new IntPtr(1), ref w);

            //EM_SETTABSTOPS - http://msdn.microsoft.com/en-us/library/bb761663%28VS.85%29.aspx
            editor.Invalidate();
        }

        public void setTabWidth2(uint tabWidth) {
            int w;
            Graphics graphics = editor.CreateGraphics();
            var characterWidth = graphics.MeasureString("m", editor.Font).Width;
            w = (int)tabWidth;
            // w = tabWidth;
            SendMessage(editor.Handle, EM_SETTABSTOPS, new IntPtr(1), ref w);

            //EM_SETTABSTOPS - http://msdn.microsoft.com/en-us/library/bb761663%28VS.85%29.aspx
            editor.Invalidate();
        }





        private ToolStripMenuItem createMn(String caption, bool clickevent=false) {
            ToolStripMenuItem mn = new ToolStripMenuItem();
            mn.Name = "mn" + caption;
            mn.Text = caption;
            if (clickevent) {
                //mn.Click += new System.EventHandler(this.help_Click);
            }
            return mn;
        }



        private void checkFile() {
            if (editor.Modified) {
                DialogResult res = Basis.readYesNoCancel("Sollen die Änderungen im Editor gespeichert werden?", "Datei laden");
                switch (res) {
                    case DialogResult.Yes: saveFile();
                        break;
                    case DialogResult.No:
                        break;
                    case DialogResult.Cancel:
                        break;
                }
            }
        }

        private void loadFilename(String filename) {
            checkFile();
            editor.Text = File.ReadAllText(filename);
            editor.Modified = false;
        }

        public void saveFile() {
            if (!_filename.Equals("")) {
                File.WriteAllText(_filename, editor.Text);
                editor.Modified = false;
            }
        }

        private void setCursor() {
            int lineindex = editor.GetFirstCharIndexOfCurrentLine();
            int linecount = Basis.getLinenr(lineindex, editor.Text);
            int columnCount = editor.SelectionStart - lineindex + 1;
            this.lRow.Text = "Zl: " + linecount;
            this.lColumn.Text = "Sp: " + columnCount;
        }

        private void editor_TextChanged(object sender, EventArgs e) {
            setCursor();
        }

        private void editor_MouseDown(object sender, MouseEventArgs e) {
            setCursor();
        }

        private void editor_KeyPress(object sender, KeyPressEventArgs e) {
            setCursor();
        }


        private void mnUndo_Click(object sender, EventArgs e) {
            editor.Undo();
        }

        private void mnCut_Click(object sender, EventArgs e) {
            editor.Cut();
        }

        private void mnCopy_Click(object sender, EventArgs e) {
            editor.Copy();
        }

        private void mnPaste_Click(object sender, EventArgs e) {
            editor.Paste();
        }

        private void mnSelectAll_Click(object sender, EventArgs e) {
            editor.SelectAll();
        }

        public void setColors() {
            editor.ForeColor = ConstantsVariablen.Fontcolor;
            editor.BackColor = ConstantsVariablen.BGcolor;
            flowLayoutPanel1.BackColor = ConstantsVariablen.BGcolor;
            lRow.ForeColor = ConstantsVariablen.Fontcolor;
            lColumn.ForeColor = ConstantsVariablen.Fontcolor;
            lDateiname.ForeColor = ConstantsVariablen.Fontcolor;
        }


        private void WEditor_FormClosing(object sender, FormClosingEventArgs e) {
            if (editor.Modified) {
                DialogResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?" + "\r\nDateiname: " + _filename, "Programm beenden");
                switch (res) {
                    case DialogResult.Yes: saveFile();
                        break;
                    case DialogResult.No:
                        break;
                    case DialogResult.Cancel: e.Cancel = true;
                        break;
                }
            }
            parent.CloseWindow(this);
        }

        private void insertText(String text) {
            Clipboard.SetDataObject(text);
            editor.Paste();
            editor.Modified = true;

            //int lineindex = editor.GetFirstCharIndexOfCurrentLine();
            //int linecount = Basis.getLinenr(lineindex, editor.Text);
            //int columnCount = editor.SelectionStart - lineindex + 1;
            //int startindex = lineindex + linecount;
            //if (startindex >= editor.Text.Length) {
            //    editor.AppendText(text );
            //    editor.Modified = true;
            //}
            //else {
            //    int endeLine = editor.Text.IndexOf("\r\n", lineindex + linecount);
            //    editor.Text = editor.Text.Insert(endeLine + 2, text);
            //    editor.Modified = true;
            //}
        }

        private void mnSearch_Click(object sender, EventArgs e) {
            WSearch dialog = new WSearch(this);
            if (dialog.ShowDialog() == DialogResult.OK) {
                search = dialog.SearchString;
                ignoreCase = dialog.IgnoreCase;
                searchText(search, ignoreCase);
            }
        }

        private void mnSearchAgain_Click(object sender, EventArgs e) {
            searchAgain();
        }


        public void searchText(String search, Boolean ignoreCase) {
            this.search = search;
            this.ignoreCase = ignoreCase;
            position = 0;
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0) {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;
                editor.ScrollToCaret();
                editor.Invalidate();
            }
            else {
                MessageBox.Show("Die Suche war nicht erfolgreich.\r\nSuchwert: " + search,
                    "Suchen", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
        }

        public void searchAgain() {
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0) {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;
                editor.ScrollToCaret();
                editor.Invalidate();
            }
        }




        // cmd /k D:\p\Musterklausur\mk_ui_add.py
        internal void start(bool withCMD) {
            if (_filename.Length == 0) {
            }
            saveFile();
            Process Prog = new Process();
            if (withCMD) {
                Prog.StartInfo.FileName = "cmd.exe";
                Prog.StartInfo.Arguments = "/k " + _filename;
            }
            else {
                Prog.StartInfo.FileName = _filename;
                Prog.StartInfo.Arguments = "";
            }
            Prog.StartInfo.WorkingDirectory = Basis.getPath(_filename);
            Prog.Start();
            //int i = Prog.ExitCode;
            //i = Prog.ExitCode;
        }

        private void mnIf_Click(object sender, EventArgs e) {
            insertText("\tif a == 0:\r\n\t\t");
        }

        private void mnIfElse_Click(object sender, EventArgs e) {
            insertText("\tif a == 0:\r\n\t\tpass\r\n\telse:\r\n\t\tpass");
        }

        private void mnIfElifElse_Click(object sender, EventArgs e) {
            insertText("\tif a == 0:\r\n\t\tpass\r\n\telif b == 0:\r\n\t\tpass\r\n\telse:\r\n\t\tpass");
        }

        private void mnWhile_Click(object sender, EventArgs e) {
            insertText("\twhile True:\r\n\t\tpass");
        }

        private void mnForEach_Click(object sender, EventArgs e) {
            insertText("\tfor i in [1,2,3]:\r\n\t\tprint(i)");
        }

        private void mnForRange_Click(object sender, EventArgs e) {
            insertText("\tfor i in range(1,10,2):\r\n\t\tprint(i)");
        }

        private void mnInt2String_Click(object sender, EventArgs e) {
            insertText("\tiValue=str(123)");
        }

        private void mnDouble2String_Click(object sender, EventArgs e) {
            insertText("\tdValue=str(123.4)");
        }

        private void mnString2Int_Click(object sender, EventArgs e) {
            insertText("\tiValue=int('123')");
        }

        private void mnString2Float_Click(object sender, EventArgs e) {
            insertText("\tfValue=float('123.4')");
        }

        private void mnShowType_Click(object sender, EventArgs e) {
            insertText("\tprint('type(var1): ', type(var1))");
        }

       

        private void mnFunktionVoid_Click(object sender, EventArgs e) {
            insertText("\tdef fktname(param1):\r\n\t\tprint(param1)");
        }

        private void mnFunktionInt_Click(object sender, EventArgs e) {
            insertText("\tdef fktname(value):\r\n\t\tprint(value)\r\n\treturn value+1");
        }

        private void mnFunktionTuple_Click(object sender, EventArgs e) {
            insertText("def fktname(value):\r\n\t\tprint(param)\r\n\t# Abfrage: t[0], t[1]\r\n\treturn (value+1, value+2)");
        }

        private void mnList1_Click(object sender, EventArgs e) {
            insertText(
                "# einfache Liste\r\n" +
                "liste = [1,2,13,13,35,6,7,8,9]\r\n" +
                "print(\"Ausgabe mit foreach:\")\r\n" +
                "for item in liste:\r\n" +
                "\tprint(item)\r\n" +
                "print()\r\n" +
                "print(\"Ausgabe mit for range:\")\r\n" +
                "for i in range(0,len(liste),1):\r\n" +
                "\tprint( liste[i] )\r\n" +
                "Methoden\r\n" +
                "# Anzahl der Elemente in der Liste: len(liste) \r\n" +
                "# Min-Element: min(liste) \r\n" +
                "# Max-Element: max(liste) \r\n" +
                "# Suche 6: liste.count(6) \r\n" +
                "# Position von 13: liste.index(13) \r\n" +
                "# insert: liste.insert(42,1)\r\n" +
                "# liste.reverse()\r\n"
                );
        }

        private void mnDict1_Click(object sender, EventArgs e) {
            insertText(
                "noten = {27456:1.3 , 27457:3.3 , 27459:2.7 , \"Brücke\":\"Extrakäse\" } \r\n" +
                "for key in noten: \r\n" +
                "\tprint(key) \r\n" +
                "print(\"Ausgabe mit einem Key:\") \r\n" +
                "print( noten[27456] ) \r\n" +
                "for key in noten.keys(): \r\n" +
                "\tprint(key, \"  \",noten[key]) \r\n" +
                "print(\"Anzahl der Elemente in der Dictionary\", len(noten) ) \r\n"
            );
        }

        private void mnSet1_Click(object sender, EventArgs e) {
            insertText(
                "s = set( (2,3,5,7) ) \r\n" +
                "for item in s: \r\n" +
                "\tprint(item) \r\n\r\n" +
                "print() \r\n" +
                "s.add(\"Pizza\") \r\n" +
                "s.add(\"Pizza\") \r\n\r\n" +
                "for item in s: \r\n" +
                "\tprint(item) \r\n\r\n" +
                "print() \r\n" +
                "print(len(s)) \r\n"
            );
        }

        // ValueError
        // IndexError
        // ArithmeticError
        // EOFError
        // IOError
        // AttributeError  hat kein Instanz hat kein Attribut
        // MemoryError
        // OverflowError
        // ZeroDivisionError
        private void mnExceptionCast_Click(object sender, EventArgs e) {
            insertText(
                "try:\r\n" +
                "\ta = int('abc')\r\n" +
                "except ValueError as e:\r\n" +
                "\tprint(\"fehlerhafte Zahl\")\r\n\r\n" +
                "\t# IndexError\r\n" +
                "\t# ArithmeticError\r\n" +
                "\t# EOFError\r\n" +
                "\t# IOError\r\n" +
                "\t# AttributeError  hat kein Instanz hat kein Attribut\r\n" +
                "\t# MemoryError\r\n" +
                "\t# OverflowError\r\n" +
                "\t# ZeroDivisionError\r\n\r\n" +
                "# raise ValueError('represents a hidden bug, do not catch this')"
            );
        }

        private void mnExceptionFull_Click(object sender, EventArgs e) {
            insertText(
                "try:\r\n" +
                "\ta = int('abc')\r\n" +
                "except ValueError as e1:\r\n" +
                "\tprint(\"fehlerhafte Zahl\")\r\n\r\n" +
                "except OverflowError as e2:\r\n" +
                "\tprint(\"overflowError\")\r\n\r\n" +
                "else:\r\n" +
                "\t#ausgefuehrt, wenn keine Exception erzeugt wurde\r\n" +
                "\tpass\r\n" +
                "finally:\r\n" +
                "\t#immer ausgefuehrt\r\n" +
                "\tpass\r\n\r\n" +
                "# raise ValueError('represents a hidden bug, do not catch this')"

            );
        }

        private void mnSimpleClass_Click(object sender, EventArgs e) {
            insertText("class Punkt:\r\n" +
                           "\tdef __init__(self, x, y):\r\n" +
                           "\t\tself.x=x\r\n" +
                           "\t\tself.y=y\r\n\r\n" +
                           "\tdef __str__(self):\t# toString\r\n" +
                           "\t\treturn str(self.x)+\", \"+str(self.y)" + "\r\n" +
                           "# main\r\n" +
                           "p1 = Punkt(12,3)\r\n" +
                           "p1.x=42\r\n" +
                           "print(p1.x,p1.y)\r\n");
        }

        private void mnSimpleClassSetGetter_Click(object sender, EventArgs e) {
            insertText("class Punkt:\r\n" +
                           "\tdef __init__(self, x):\r\n" +
                           "\t\tself._X=x\r\n\r\n" +
                           "\tdef getX(self):\r\n" +
                           "\t\treturn self._X\r\n\r\n" +
                           "\tdef setX(self,x):\r\n" +
                           "\t\tself._X=x\r\n\r\n" +
                           "# main\r\n" +
                           "p1 = Punkt(12)\r\n" +
                           "p1.setX(42)\r\n" +
                           "print(p1.getX())\r\n");
        }

        private void mnSimple2Class_Click(object sender, EventArgs e) {
            insertText("class Schiff:\r\n" +
                        "\tdef __init__(self, name):\r\n\r\n" +
                        "\t\tself._name=name\r\n" +
                        "\tdef getName(self):\r\n" +
                        "\t\treturn self._name\r\n\r\n" +
                        "\tdef setName(self,name):\r\n" +
                        "\t\tself._name=name\r\n\r\n" +

                        "class Frachter(Schiff):\r\n\r\n" +
                        "\tdef __init__(self, name, frachtvolumen):\r\n" +
                        "\t\tSchiff.__init__(self, name)\r\n" +
                        "\t\tself._frachtvolumen=frachtvolumen\r\n\r\n" +
                        "\tdef getFrachtvolumen(self):\r\n" +
                        "\t\treturn self._frachtvolumen\r\n\r\n" +
                        "\tdef setFrachtvolumen(self,frachtvolumen):\r\n" +
                        "\t\tself._frachtvolumen=frachtvolumen\r\n\r\n" +
                        "\tdef __str__(self):\r\n" +
                        "\t\treturn Schiff.getName(self)+\"  \"+str(self.getFrachtvolumen())\r\n\r\n" +
                        "# main\r\n" +
                        "s1 = Schiff(\"Titanic\")\r\n" +
                        "s1.setName(\"Queen Mary\")\r\n" +
                        "print(s1.getName())\r\n" +
                        "f1 = Frachter(\"KaesePizza\", 12000)\r\n" +
                        "f1.setFrachtvolumen(14000)\r\n" +
                        "print(f1)\r\n");
        }

        private void mnInputString_Click(object sender, EventArgs e) {
            insertText("inp = input(\"Geben Sie bitte keinen Text ein\")\r\nprint(inp)");
        }

        private void mnInputInt_Click(object sender, EventArgs e) {
            insertText("zahl = int(input(\"Geben Sie bitte eine Zahl ein\"))\r\nprint(zahl)");
        }

        private void mnInputFloat_Click(object sender, EventArgs e) {
            insertText("zahl = float(input(\"Geben Sie bitte eine Zahl ein\"))\r\nprint(zahl)");
        }

        private void mnArgv1_Click(object sender, EventArgs e) {
            insertText("import sys\r\n\r\n" +
                        "print(len(sys.argv))\r\n\r\n" +
                        "if len(sys.argv)>0:\r\n" +
                        "\t# kompletter Name des Skriptes\r\n" +
                        "\tprint(sys.argv[0])\r\n\r\n"
            );
        }


        private void mnImportOs_Click(object sender, EventArgs e) {
            insertText("import os\r\n");
        }

        private void mnImportOsPath_Click(object sender, EventArgs e) {
            insertText("import os.path\r\n");
        }

        private void getCWD_Click(object sender, EventArgs e) {
          insertText("\t\tpath = getcwd()\r\n");
        }

        private void mnListdir_Click(object sender, EventArgs e) {
            insertText("\t\titems = os.listdir(path)\r\n");
        }

        private void mnIsdir_Click(object sender, EventArgs e) {
            insertText("\t\tvalue = os.path.isdir(item)\r\n");
        }

        private void mnIsfile_Click(object sender, EventArgs e) {
            insertText("\t\tvalue = os.path.isfile(item)\r\n");
        }



        private void mnUIFramework_Click(object sender, EventArgs e) {
            insertText(
                "import tkinter\r\n\r\n" +
                "class MyApp(tkinter.Frame):\r\n\r\n" +
                "\tdef __init__(self, master=None):\r\n" +
                "\t\ttkinter.Frame.__init__(self, master)\r\n" +
                "\t\tself.pack(expand=True, fill=\"both\") \r\n" +
                "\t\tself.setGUI()\r\n\r\n" +
                "\tdef setGUI(self):\r\n" +
                "\t\tself.inputui = tkinter.Entry(self)\r\n" +
                "\t\tself.inputui.pack()\r\n" +
                "\t\tself.bnEsc = tkinter.Button(self)\r\n" +
                "\t\tself.bnEsc[\"text\"] = \"Ende\"\r\n" +
                "\t\tself.bnEsc[\"command\"] = self.quit\r\n" +
                "\t\tself.bnEsc.pack(side=\"right\")\r\n\r\n" +

                "root = tkinter.Tk()\r\n" +
                "root.title(\"Mein Fenster\")\r\n" +
                "root.geometry(\"250x100\")\r\n" +
                "app = MyApp(root)\r\n" +
                "app.mainloop()\r\n"
            );
        }

        private void mnUIFrame_Click(object sender, EventArgs e) {
            insertText(
                "\t\tframe1 = tkinter.Frame(self)\r\n" +
                "\t\tframe1.pack(fill=\"x\", side=\"top\" ) \t # fill=x,y,both\r\n"
            );
        }

        private void mnLabelframe_Click(object sender, EventArgs e) {
            insertText(
                "\t\tlabelframe = tkinter.LabelFrame(self, text=\"Caption\")\r\n" +
                "\t\tlabelframe.pack(fill=\"both\", expand=\"yes\")\r\n" 
            );

        }

        private void mnScrollbarVertical_Click(object sender, EventArgs e) {
            insertText(
                "\t\tsby = tkinter.Scrollbar(self)\r\n" +
                "\t\tsby.pack(fill=\"y\", side=\"right\")\r\n\r\n" +
                "\t\t# UI-Element\r\n" +
                "\t\t# self.element[\"yscrollcommand\"] = sbx.set\r\n" +
                "\t\t#sby[\"command\"] = self.element.yview\r\n"
            );
        }

        private void mnScrollbarHorizontal_Click(object sender, EventArgs e) {
            insertText(
                "\t\tsbx = tkinter.Scrollbar(self, orient=\"horizontal\")\r\n" +
                "\t\tsbx.pack(fill=\"x\", side=\"bottom\")\r\n\r\n" +

                "\t\t# UI-Element\r\n" +

                "\t\t# self.element[\"xscrollcommand\"] = sbx.set\r\n" +
                "\t\tsbx[\"command\"] = self.element.xview\r\n" 
            );
        }

        private void mnUILabel_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.label1 = tkinter.Label(self)\r\n" +
                "\t\tself.label1[\"text\"] = \"Eingabe\"\r\n" +
                "\t\tself.label1.pack()\r\n"
            );
        }

        private void mnUITextbox_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.inputui = tkinter.Entry(self)\r\n" +
                "\t\tself.inputui.pack(fill=\"x\",padx=\"5\",pady=\"5\")\r\n"
            );
        }

        private void mnUISpinbox_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.spinbox = tkinter.Spinbox(inputframe, from_=0, to=10)\r\n" +
                "\t\t# self.spinbox = tkinter.Spinbox(inputframe,values = (\"AI\",\"VW\",\"W\"))\r\n" +
                "\t\tself.spinbox.pack(fill=\"x\",padx=\"5\",pady=\"5\")\r\n"
            );
        }

        private void mnUIButton_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.bnOk = tkinter.Button(self)\r\n" +
                "\t\tself.bnOk[\"text\"] = \"Ok\"\r\n" +
                "\t\tself.bnOk[\"command\"] = self.quit\r\n" +
                "\t\tself.bnOk.pack(side=\"right\")\r\n"

            );
        }

        private void mnUIRadiobutton_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.group = tkinter.LabelFrame(self, text=\"Noten\")\r\n" +
                "\t\tself.group.pack(fill=\"both\", expand=\"yes\")\r\n" +
                "\t\tself.rb1= tkinter.Radiobutton(self.group, text=\"Eins\", value=\"Eins\")\r\n" +
                "\t\tself.rb1.pack()\r\n" +
                "\t\tself.rb2= tkinter.Radiobutton(self.group, text=\"Zwei\", value=\"Zwei\")\r\n" +
                "\t\tself.rb2.pack()\r\n" +
                "\t\tself.rbProperty = tkinter.StringVar()\r\n" +
                "\t\tself.rbProperty.set(\"Zwei\")\r\n" +
                "\t\tself.rb1[\"variable\"] = self.rbProperty\r\n" +
                "\t\tself.rb2[\"variable\"] = self.rbProperty\r\n"
            );
        }

        private void mnUICheckbutton_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.group = tkinter.LabelFrame(self, text=\"Noten\")\r\n" +
                "\t\tself.group.pack(fill=\"both\", expand=\"yes\")\r\n" +
                "\t\tself.chk1= tkinter.Checkbutton(self)\r\n" +
                "\t\tself.chk1[\"text\"] = \"Eins\"\r\n" +
                "\t\tself.chk1.pack()\r\n" +
                "\t\tself.chk2= tkinter.Checkbutton(self)\r\n" +
                "\t\tself.chk2[\"text\"] = \"Zwei\"\r\n" +
                "\t\tself.chk2.pack()\r\n" +
                "\t\tself.chkProperty1 = tkinter.BooleanVar()\r\n" +
                "\t\tself.chkProperty1.set(True)\r\n" +
                "\t\tself.chk1[\"variable\"] = self.chkProperty1\r\n" +
                "\t\tself.chkProperty2 = tkinter.BooleanVar()\r\n" +
                "\t\tself.chkProperty2.set(False)\r\n" +
                "\t\tself.chk2[\"variable\"] = self.chkProperty2\r\n"
            );
        }

        private void mnUIListbox_Click(object sender, EventArgs e) {
            insertText(
                "\t\tsb = tkinter.Scrollbar(self)\r\n" +
                "\t\tsb.pack(fill=\"y\", side=\"right\")\r\n" +
                "\t\tself.listbox = tkinter.Listbox(self)\t # selectmode=\"single, extended orient=\"horizontal\"   \"\r\n" +
                "\t\tself.listbox.pack(expand=True, fill=\"both\")\r\n" +
                "\t\tself.listbox[\"yscrollcommand\"] = sb.set\r\n" +
                "\t\tsb[\"command\"] = self.listbox.yview\r\n" +
                "\t\tself.listbox.insert(\"end\", \"abc\")\r\n"
            );
        }

        private void mnUIEditor_Click(object sender, EventArgs e) {
            insertText(
            "\t\tsbx = tkinter.Scrollbar(self, orient=\"horizontal\")\r\n" +
            "\t\tsbx.pack(fill=\"x\", side=\"bottom\")\r\n\r\n" +

            "\t\tsby = tkinter.Scrollbar(self)\r\n" +
            "\t\tsby.pack(fill=\"y\", side=\"right\")\r\n\r\n" +

            "\t\tself.editor = tkinter.Text(self)\r\n" +
            "\t\tself.editor.config(wrap=\"none\")  # wrap=\"word\"  word char \r\n" +
            "\t\tself.editor.pack(expand=True, fill=\"both\", padx=\"5\",pady=\"5\")\r\n\r\n" +

            "\t\tself.editor[\"xscrollcommand\"] = sbx.set\r\n" +
            "\t\tsbx[\"command\"] = self.editor.xview\r\n" +
            "\t\tself.editor[\"yscrollcommand\"] = sby.set\r\n" +
            "\t\tsby[\"command\"] = self.editor.yview\r\n"
            );
        }

        private void mnUIStringVar_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.var_inputfield1 = tkinter.StringVar()\r\n" +
                "\t\tself.var_inputfield1.set(\"abc\")\r\n" +
                "\t\tself.inputui1[\"textvariable\"] = self.var_inputfield1\r\n"
            );
        }

        private void mnUIIntVar_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.var_inputfield1 = tkinter.IntVar()\r\n" +
                "\t\tself.var_inputfield1.set(12)\r\n" +
                "\t\tself.inputui1[\"textvariable\"] = self.var_inputfield1\r\n"
            );
        }

        private void mnUIDoubleVar_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.var_inputfield1 = tkinter.DoubleVar()\r\n" +
                "\t\tself.var_inputfield1.set(12.5)\r\n" +
                "\t\tself.inputui1[\"textvariable\"] = self.var_inputfield1\r\n"
            );
        }

        private void mnUIBooleanVar_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.chkProperty1 = tkinter.BooleanVar()\r\n" +
                "\t\tself.chkProperty1.set(True)\t# self.chkProperty1.get()\r\n" +
                "\t\tself.inputui1[\"textvariable\"] = self.chkProperty1\r\n"
            );
        }

        private void mnUIListBoxInsert_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.listbox.insert(0,\"abc\")\r\n"
            );
        }

        private void mnUIListBoxAdd_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.listbox.insert(\"end\",\"abc\")\r\n"
            );
        }

        private void mnUIListBoxSelect_Click(object sender, EventArgs e) {
            insertText(
                "\t\tindizes = self.listbox.curselection()\t # tupel, selectmode=extended\r\n" +
                "\t\tindex = indizes[0]\r\n"
            );
        }

        private void mnUIListBoxGet_Click(object sender, EventArgs e) {
            insertText(
                "\t\titem = self.listbox.get(index)\r\n"
            );
        }

        private void mnUIListBoxDelete_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.listbox.delete(0,\"end\")\r\n"
            );
        }

        private void mnUITextInsert_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.editor.insert(\"end\",\"abc\")\r\n"
            );
        }

        private void mnUITextDelete_Click(object sender, EventArgs e) {
            insertText(
                "\t\tself.editor.delete(1.0,\"end\")\r\n"
            );
        }

        private void mnUIMsgOkCancel_Click(object sender, EventArgs e) {
            insertText(
                "\t\tmessagebox.askokcancel( \"Meldung\", \"Ihre Festplatte wird formatiert\")\r\n" +
                "\t\t    # icon='question', error, warning, info, question\r\n"
            );
        }

        private void mnUIMsgYesNo_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = messagebox.askyesno(\"Delete\", \"Are You Sure?\", \r\n" +
                "\t\t\ticon='question', default='no')\r\n" +
                "\t\tif result == True:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Deleted\",\"Deleted\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Not Deleted\",\"Not Deleted\")\r\n" +
                "\t\t    # icon='question', error, warning, info, question\r\n"
            );
        }

        private void mnUIMsgYesNoCancel_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = messagebox.askyesnocancel( \"Beenden\", \"Änderungen speichern?\")\r\n" +
                "\t\tif result == True:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Beenden\",\"Save\")\r\n" +
                "\t\telif result == False:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Beenden\",\"No Save\")\r\n" +
                "\t\telif result == None:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Beenden\",\"No Close\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Beenden\",\"Fehlerhafte Abfrage\")\r\n" +
                "\t\t    # icon='question', error, warning, info, question\r\n\r\n"
            );
        }


        private void mnUIMsgRetryCancel_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = messagebox.askretrycancel( \"Datei öffnen\", \"Fehler beim Öffnen, noch einmal versuchen?\")\r\n" +
                "\t\tif result == True:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Datei öffnen\",\"Noch einmal\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Datei öffnen\",\"Ende, kein weiterer Versuch\")\r\n" +
                "\t\t\t # messagebox.askokcancel( \"Note\", \"Sie erhalten eine 1,0\", default='yes')\r\n"
            );
        }

        private void mnUIAskString_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = simpledialog.askstring( \"Hello Python\", \"Hello World\")\r\n" +
                "\t\tif result == None:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",\"keine\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",result)\r\n"
            );
        }

        private void mnUIAskInt_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = simpledialog.askinteger( \"Hello Python\", \"Hello World\", initialvalue=42, minvalue=1, maxvalue=99)\r\n" +
                "\t\tif result == None:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",\"keine\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",result)\r\n"
            );
        }

        private void mnUIAskFloat_Click(object sender, EventArgs e) {
            insertText(
                "\t\tresult = simpledialog.askfloat( \"Hello Python\", \"Hello World\", initialvalue=42.0, minvalue=1.0, maxvalue=99.9)\r\n" +
                "\t\tif result == None:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",\"keine\")\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",result)\r\n"
            );
        }

        private void mnUIOpenDialog_Click(object sender, EventArgs e) {
            insertText(
                "\t\tfilename = filedialog.askopenfilename(title=\"Dateien laden\",\r\n" +
                "\t\t\t initialdir=\"D:/Daten\", multiple=False,\r\n" +
                "\t\t\t filetypes=( (\"Pythondateien\",\".py\" ),(\"Alle Dateien\",\"*.*\" ) ) )\r\n" +
                "\t\tif filename:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Eingabe\",filename)\r\n\r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Auswahl\",\"Abbruch\")\r\n"
            );
        }

        private void mnUISaveDialog_Click(object sender, EventArgs e) {
            insertText(
                "\t\tfilename = filedialog.asksaveasfilename(initialfile=\"Beispiel.py\", \r\n" +
                "\t\t\t title=\"Dateien laden\", \r\n" +
                "\t\t\t initialdir=\"D:\\Python\\\",\r\n" +
                "\t\t\t filetypes=( (\"Pythondateien\",\".py\" ),(\"Alle Dateien\",\"*.*\" ) ) )\r\n" +
                "\t\tif filename:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Auswahl\",filename)   \r\n" +
                "\t\telse:\r\n" +
                "\t\t\tmessagebox.showinfo(\"Auswahl\",\"Abbruch\")\r\n"
            );
        }

        private void mnAbs_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = abs(-12)\r\n"
            );
        }

        private void mnFabs_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.fabs(-12.5)\r\n"
            );
        }

        private void mnMin_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = min([1,2,3])\r\n"
            );
        }

        private void mnMax_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = max([1,2,3])\r\n"
            );
        }

        private void mnExp_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.exp(1)\r\n"
            );
        }

        private void mnLog10_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.log10(11)\r\n"
            );
        }

        private void mnLogxbase_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.log(100,10)\t ==> 2\r\n"
            );
        }

        private void mnLn_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.log(11)\t  ln(11) ==> 2.3978952727983707\r\n"
            );
        }

        private void mnSqrt_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.sqrt(2)\r\n"
            );
        }

        private void mnFloor_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.floor(-12.2)\t  => -13\r\n"+
                "\t\tvar1 = math.floor(12.2)\t  => 12\r\n"
            );
        }

        private void mnTrunc_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = math.floor(-12.2)\t  => -12\r\n" +
                "\t\tvar1 = math.floor(12.2)\t  => 12\r\n"
            );
        }

        private void mnRound_Click(object sender, EventArgs e) {
            insertText(
                "\t\tvar1 = round(12.2)\t  => 12\r\n" +
                "\t\tvar1 = round(12.5)\t  => 12\r\n" +
                "\t\tvar1 = round(12.6)\t  => 13\r\n"+
                "\t\tvar1 = round(12.66,1)\t  => 12.7\r\n"
            );
        }

        private void mnUIForeground_Click(object sender, EventArgs e) {
            insertText(
                "\t\tuiElement.config(fg=\"red\")\r\n"
            );
        }

        private void mnUIBackground_Click(object sender, EventArgs e) {
            insertText(
                "\t\tuiElement.config(bg=\"yellow\")\r\n"
            );
        }

        private void anchorToolStripMenuItem_Click(object sender, EventArgs e) {
            // N, NE, E, SE, S, SW, W, NW, or CENTER
            insertText(
                "\t\tuiElement.config(anchor=\"n\")\t# n, ne, e, se, s, sw, w, wn, center\r\n"
            );
        }

        // justify=tkinter.RIGHT
        private void justifyAusrichtungToolStripMenuItem_Click(object sender, EventArgs e) {
            insertText(
                "\t\tuiElement.config(justify=\"tkinter.LEFT\")\t# LEFT, RIGHT, or CENTER\r\n"
            );
        }

    }
}
